import graphene
from graphene_django import DjangoObjectType
from .models import Payer
from location.schema import LocationGQLType
from core import prefix_filterset, filter_validity, ExtendedConnection

class PayerGQLType(DjangoObjectType):
    class Meta:
        model = Payer
        interfaces = (graphene.relay.Node,)
        filter_fields = {
            "id": ["exact"],
            "uuid": ["exact"],
            "name": ["exact", "icontains"],
        }
        connection_class = ExtendedConnection



from django.db.models import Q
from django.core.exceptions import PermissionDenied
from graphene_django.filter import DjangoFilterConnectionField
import graphene_django_optimizer as gql_optimizer

from django.utils.translation import gettext as _
from location.apps import LocationConfig
from core.schema import OrderedDjangoFilterConnectionField

# We do need all queries and mutations in the namespace here.
from .gql_queries import *  # lgtm [py/polluting-import]
from .gql_mutations import *  # lgtm [py/polluting-import]

class Query(graphene.ObjectType):
    payers = OrderedDjangoFilterConnectionField(
        PayerGQLType,
        parent_location=graphene.String(),
        parent_location_level=graphene.Int(),
        orderBy=graphene.List(of_type=graphene.String),
    )

    def resolve_payers(self, info, **kwargs):
        # if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
        #     raise PermissionDenied(_("unauthorized"))
        filters = []
        # show_history = kwargs.get('show_history', False)
        # if not show_history and not kwargs.get('uuid', None):
        #     filters += filter_validity(**kwargs)
        parent_location = kwargs.get('parent_location')
        if parent_location is not None:
            parent_location_level = kwargs.get('parent_location_level')
            if parent_location_level is None:
                raise NotImplementedError("Missing parentLocationLevel argument when filtering on parentLocation")
            f = "uuid"
            for i in range(len(LocationConfig.location_types) - parent_location_level - 1):
                f = "parent__" + f
            payer_location = "location__" + f
            filters.append(Q(**{payer_location: parent_location}))
        return gql_optimizer.query(Payer.objects.filter(*filters).all(), info)



import uuid

from core import fields
from core import models as core_models
from django.db import models


class PayerType(models.Model):
    code = models.CharField(db_column='Code', primary_key=True, max_length=1)
    payer_type = models.CharField(db_column='PayerType', max_length=50)
    alt_language = models.CharField(db_column='AltLanguage', max_length=50, blank=True, null=True)
    sort_order = models.IntegerField(db_column='SortOrder', blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'tblPayerType'


class Payer(core_models.VersionedModel):
    PAYER_TYPE_COOP = 'C'
    PAYER_TYPE_DONOR = 'D'
    PAYER_TYPE_GOV = 'G'
    PAYER_TYPE_LOCAL_AUTH = 'L'
    PAYER_TYPE_OTHER = 'O'
    PAYER_TYPE_PRIVATE_ORG = 'P'
    PAYER_TYPE_CHOICES = (
        (PAYER_TYPE_COOP, 'Co-operative'),
        (PAYER_TYPE_DONOR, 'Donor'),
        (PAYER_TYPE_GOV, 'Government'),
        (PAYER_TYPE_LOCAL_AUTH, 'Local Authority'),
        (PAYER_TYPE_OTHER, 'Other'),
        (PAYER_TYPE_PRIVATE_ORG, 'Private Organization'),
    )

    id = models.AutoField(db_column='PayerID', primary_key=True)
    uuid = models.CharField(db_column='PayerUUID', max_length=36, default=uuid.uuid4, unique=True)
    type = models.CharField(db_column='PayerType', max_length=1, choices=PAYER_TYPE_CHOICES)
    # payer_type = models.ForeignKey(
    #     "contribution.PayerType", models.DO_NOTHING, db_column='PayerType', blank=True, null=True)
    name = models.CharField(db_column='PayerName', max_length=100, null=False)
    address = models.CharField(db_column='PayerAddress', max_length=100, null=True, blank=True)
    location = models.ForeignKey("location.Location", db_column="LocationId", blank=True, null=True,
                                 on_delete=models.DO_NOTHING, related_name='+')
    phone = models.CharField(db_column='Phone', max_length=50, null=True, blank=True)
    fax = models.CharField(db_column='Fax', max_length=50, null=True, blank=True)
    email = models.CharField(db_column='eMail', max_length=50, null=True, blank=True)

    audit_user_id = models.IntegerField(db_column='AuditUserID')

    # rowid = models.TextField(db_column='RowID', blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'tblPayer'



import uuid

from contribution.models import Payer, Premium


def create_test_payer(payer_type=Payer.PAYER_TYPE_OTHER, custom_props=None):
    payer = Payer.objects.create(
        **{
            "type": payer_type,
            "uuid": uuid.uuid4(),
            "name": "Test Default Payer Name",
            "address": "Test street name 123, CZ9204 City, Country",
            "validity_from": "2019-01-01",
            "audit_user_id": -1,
            **(custom_props if custom_props else {})
        }
    )
    return payer
